@@ -11,20 +11,25 @@ import pandas as pd
 import pickle5 as pickle
 from scipy import interpolate
 
-def get_convergence(search_dir,with_surrogate=False,sigma=False,max_iter=None,**kwargs):
-    dir_list = glob.glob(search_dir+'/*')
+def get_convergence(search_dir,with_surrogate=False,sigma=False,niters=None,nruns=None,debug=False,**kwargs):
+    dir_list = sorted(glob.glob(search_dir+'/*'))
     if any('.log' in dir_name for dir_name in dir_list):
         dir_list = [search_dir]
     else:
         dir_list = [os.path.join(dir_name,'logs') for dir_name in dir_list]
+    n = 0
     all_results = [[],[]]
     for dir_name in dir_list:
         if os.path.isdir(dir_name)==False:
             continue
+        if nruns!=None and n==nruns:
+            break
         log_files = '%s/evaluation*_01.log' % dir_name
-        evaluation = extract(log_files,**kwargs).sort_values(by=['nevals']).to_numpy()[:,1:3]
+        if debug:
+            print('Searching',log_files)
+        evaluation = extract(log_files,debug=debug,**kwargs).sort_values(by=['nevals']).to_numpy()[:,1:3]
         if with_surrogate:
-            surrogate  = extract(log_files.replace('evaluation','surrogate'),**kwargs).sort_values(by=['nevals']).to_numpy()[:,1:3]
+            surrogate  = extract(log_files.replace('evaluation','surrogate'),debug=debug,**kwargs).sort_values(by=['nevals']).to_numpy()[:,1:3]
             evaluation = numpy.vstack((evaluation,surrogate))
         loss_surr, sdev_surr = [], []
         for loss,std in evaluation:
@@ -36,29 +41,38 @@ def get_convergence(search_dir,with_surrogate=False,sigma=False,max_iter=None,**
                 sdev_surr.append(sdev_surr[-1])
         all_results[0].append(loss_surr)
         all_results[1].append(sdev_surr)
+        n+=1
+    assert len(all_results[0])>0, 'No results found. Abort.'
     nevals = [len(results) for results in all_results[0]]
-    max_iter = min(nevals) if max_iter==None or max_iter>min(nevals) else max_iter
+    niters = min(nevals) if niters==None or niters>min(nevals) else niters
     for i in range(len(all_results[0])):
-        all_results[0][i] = all_results[0][i][:max_iter]
-        all_results[1][i] = all_results[1][i][:max_iter]
+        all_results[0][i] = all_results[0][i][:niters]
+        all_results[1][i] = all_results[1][i][:niters]
     all_results = numpy.array(all_results)
     mean_loss = numpy.mean(all_results[0],axis=0)
     if sigma:
-        mean_sdev = numpy.std(all_results[0],axis=0)
-    else:
         mean_sdev = numpy.sqrt(numpy.sum(all_results[1]**2,axis=0))/all_results[0].shape[0]
+    else:
+        mean_sdev = numpy.std(all_results[0],axis=0)
     return mean_loss, mean_sdev
 
-def extract(path,target_loss='inner',target_unc='STD',**kwargs):
+def extract(path,target_loss='inner',target_unc='STD',debug=False,**kwargs):
+    hps = []
     data = []
     nevals = 0
     for fname in sorted(glob.glob(path)):
-        params = None
+        if debug:
+            print('Extract results from',fname)
+        params, gen_params, dis_params = float('nan'), float('nan'), float('nan')
         content = numpy.loadtxt(fname,dtype=str,delimiter='\n')
         for i,line in enumerate(content):
             if 'Samples:' in line:
                 samples = line.split('Samples:')[-1].strip()[1:-1]
                 samples = [int(k) for k in samples.split()]
+            if 'parameters in generator' in line:
+                gen_params = int(line.split()[-4].replace(',',''))
+            if 'parameters in discriminator' in line:
+                dis_params = int(line.split()[-4].replace(',',''))
             if 'Number of parameters :' in line:
                 params = int(line.split()[-1].replace(',',''))
             if '%s Loss' % target_loss.capitalize() in line:
@@ -69,12 +83,18 @@ def extract(path,target_loss='inner',target_unc='STD',**kwargs):
                 nevals = int(line.split()[-1])+1
             if target_unc in line:
                 std = float(line.split()[-1])
-                data.append([params,loss,std]+samples+[nevals])
+                data.append([params,loss,std]+samples+[nevals,gen_params,dis_params])
+                hps.append(samples)
             if 'Lower Bound of CI' in line:
                 std = loss-float(line.split()[-1])
-                data.append([params,loss,std]+samples+[nevals])
+                data.append([params,loss,std]+samples+[nevals,gen_params,dis_params])
+                hps.append(samples)
+    if len(data)!=len(numpy.unique(hps,axis=0)):
+        print('WARNING: Some hyperparameter sets were evaluated more than once in %s (%i/%i)' % (path,len(numpy.unique(hps,axis=0)),len(data)))
     if len(data)>0:
-        data = pd.DataFrame(data=data,columns=['params','loss','stdev']+['hp%i'%(i+1) for i in range(len(samples))]+['nevals'])
+        data = pd.DataFrame(data=data,columns=['params','loss','stdev']+['hp%i'%(i+1) for i in range(len(samples))]+['nevals','gen_params','dis_params']).sort_values(by=['nevals'])
+    if debug:
+        print(data)
     return data
 
 def check_surrogate_sample(log_dir,step,iloop):
@@ -105,7 +125,7 @@ def check_evaluation_outer_loss(target,output,log_dir,step,**kwargs):
                 break
     return outer_loss
 
-def gather_trials(results,target,output,log_dir,ntasks,step,trial,split,**kwargs):
+def gather_trials(target,output,log_dir,ntasks,step,trial,split,**kwargs):
     """
     Collect trial losses for given sample set.
     
@@ -114,7 +134,6 @@ def gather_trials(results,target,output,log_dir,ntasks,step,trial,split,**kwargs
     results : :class:`dict`
       Results
     """
-    models = []
     if split=='trial':
         losses = numpy.zeros((trial,1))
     else:
@@ -130,21 +149,15 @@ def gather_trials(results,target,output,log_dir,ntasks,step,trial,split,**kwargs
             if 'TESTING' in line and list(samples)==list(target):
                 itrial = int(line.split()[-4])-1
                 for sub_line in log_content[i:]:
-                    if 'Loss' in sub_line and list(samples)==list(target):
+                    if 'Test Loss' in sub_line and list(samples)==list(target):
                         loss = float(sub_line.split()[-1])
-                        checkpoint_dir = os.path.join(log_dir,'checkpoints')
-                        path_to_model = os.path.join(checkpoint_dir,'%s.pth.tar' % output)
                         if split=='trial':
                             losses[itrial,0] = loss
-                            models.append(path_to_model)
                         else:
                             losses[itrial,rank] = loss
-                            if rank==0:
-                                models.append(path_to_model)
                         break
-    results['loss'] = [] if 0 in losses else numpy.mean(losses,axis=1)
-    results['models'] = [] if 0 in losses else models
-    return results
+    losses = [] if 0 in losses else numpy.mean(losses,axis=1)
+    return losses
 
 def extract_evals(log_dir,**kwargs):
     samples, fvals, uqvals = [], [], []
